#ifndef MESSAGE_HH
#define MESSAGE_HH

#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <string>

class message {

private:
	enum {max_size = 65535}; //64ko
	enum { hl_ = 5 };
	char data_[max_size];
	size_t head_length_;
	size_t body_length_;

public:
	message(): head_length_(0), body_length_(0){}
	const char* data() const {return data_;}
	const size_t length() const {return head_length_ + body_length_ + 2*hl_;}
	const size_t getHeadLength() const {return head_length_;}
	const size_t getBodyLength() const {return body_length_;}
	const char* getHeadPointer() const { return data_ + hl_ ;}
	const char* getBodyPointer() const { return data_ + head_length_ + 2*hl_;}
	std::string getStringHead() {return std::string(data_ + hl_, head_length_);}
	std::string getStringBody() {return std::string(data_ + head_length_ + 2*hl_, body_length_);}
	bool encode(const char *h, const char *b) {
		head_length_ = std::strlen(h); body_length_ = std::strlen(b);
		if((head_length_ + body_length_ + 2*hl_)>max_size) return false;
		char header_H[hl_ + 1] = ""; char header_B[hl_ + 1] = "";
		std::sprintf(header_H, "%5d", (int)head_length_);
		std::sprintf(header_B, "%5d", (int)body_length_);
		std::memcpy(data_, header_H, hl_);
		std::memcpy(data_ + hl_, h, head_length_);
		std::memcpy(data_ + hl_ + head_length_, header_B, hl_);
		std::memcpy(data_ + 2*hl_ + head_length_, b, body_length_);
		data_[this->length()] = '\0';
		return true;
	}
	void decode(const char *s) {
		std::memcpy(data_, s, std::strlen(s));
		char header_H[hl_ + 1] = ""; std::strncat(header_H, data_, hl_);
		head_length_ = std::atoi(header_H);
		char header_B[hl_ + 1] = ""; std::strncat(header_B, data_ + hl_ + head_length_, hl_);
		body_length_ = std::atoi(header_B);
		data_[this->length()] = '\0';
	}

};

#endif

